@@ -466,8 +466,8 @@ gtk_alignment_size_allocate (GtkWidget     *widget,
       padding_horizontal = priv->padding_left + priv->padding_right;
       padding_vertical = priv->padding_top + priv->padding_bottom;
 
-      width = allocation->width - padding_horizontal - 2 * border_width;
-      height = allocation->height - padding_vertical - 2 * border_width;
+      width  = MAX (1, allocation->width - padding_horizontal - 2 * border_width);
+      height = MAX (1, allocation->height - padding_vertical - 2 * border_width);
     
       if (width > child_requisition.width)
 	child_allocation.width = (child_requisition.width *
@@ -557,10 +557,10 @@ gtk_alignment_set_padding (GtkAlignment    *alignment,
 /**
  * gtk_alignment_get_padding:
  * @alignment: a #GtkAlignment
- * @padding_top: location to store the padding for the top of the widget, or %NULL
- * @padding_bottom: location to store the padding for the bottom of the widget, or %NULL
- * @padding_left: location to store the padding for the left of the widget, or %NULL
- * @padding_right: location to store the padding for the right of the widget, or %NULL
+ * @padding_top: (allow-none): location to store the padding for the top of the widget, or %NULL
+ * @padding_bottom: (allow-none): location to store the padding for the bottom of the widget, or %NULL
+ * @padding_left: (allow-none): location to store the padding for the left of the widget, or %NULL
+ * @padding_right: (allow-none): location to store the padding for the right of the widget, or %NULL
  *
  * Gets the padding on the different sides of the widget.
  * See gtk_alignment_set_padding ().
